package farm.ants.agent;
import farm.simulator.*;
import farm.metagent.*;
import farm.ants.plugins.*;
import java.util.*;
/**
 */
public class RandomAgent extends Agent {

    
    // Constants
    public static final String PROP_RANDOMUTILITY = "AgentRandomUtility";
    public static final String PROP_RANDOMTRACKS = "AgentRandomTracks";
    boolean computed = false;
    int slotCount = 3;
    Hashtable sensScheds = new Hashtable();

    /**
     * Constructor
     * @param metagent A handle to the metagent this agent belongs to
     * @param name The agents name
     */
    public RandomAgent(FarmMetAgent metagent, String name) {
	super(metagent, name);
 	setProperty(PROP_RANDOMUTILITY, new Float(0));
	setProperty(PROP_RANDOMTRACKS, new Integer(0));
    }


    /**
     * Called by the metagent to grant run time to this agent.
     * The SensorAgent simple pulls any incoming messages
     * off of its incoming message queue and if they are a rebing request
     * changes it local schedule and sends update messages to registered 
     * track managers.
     */
    public void pulse() {
	super.pulse();
        checkRun();

	if(!computed){
	    computed = computeRandom();
	}
	endRun();
    }


    private boolean computeRandom(){
	//create the list of sensors and some slot schedules
	Integer sensInt = (Integer) getProperty(FarmDriver.CLA_SENSORS);
	if(sensInt == null) return false;

	int sensors = sensInt.intValue();

	for(int i =0;i<sensors;i++){
	    sensScheds.put("SA"+i,new SlotSchedule());
	}

	//create the list of managers for the function call
	Integer targInt = (Integer) getProperty(FarmDriver.CLA_TARGETS);
	if(targInt == null) return false;

	int targets = targInt.intValue();
	Vector mans = new Vector();
	float total = 0.0f;
	int totTrack = 0;
	for(int i = 0;i<targets;i++){
	    checkRun();
	    String manName = "M" + i;
	    StringTokenizer tok = new StringTokenizer((String) getProperty(manName + ":res"), ",");
	    TMConflict TMC = new TMConflict(manName);
	    int count =0;
	    while(tok.hasMoreTokens() && count < 4){
		checkRun();
		String data = tok.nextToken();
		String sensName = data.substring(0, data.indexOf(" "));
		TMC.addConflict(sensName);
		count ++;
	    }
	    TMC.setNumAlts(TMC.numLocalConflicts());
	    mans.add(TMC);
	}
	
	//everyone wnats everything they can get, bastards!!!
	setStartObjs(mans);
	log.log(mans.toString(), log.LOG_INFO);

	//now randomly chose an order for the managers
	//and have them choose an assignment at their obj level
	//and assign it to the sensors

	Vector randMans = orderMans(mans);
	//build the data structs for the generate call
	for(Iterator i = randMans.iterator();i.hasNext();){
	    TMConflict TMC = (TMConflict) i.next();
	    Hashtable fSensInfo= new Hashtable();
	    for(Iterator j = TMC.getConflicts().iterator();j.hasNext();){
		fSensInfo.put(j.next(), new SlotSchedule());
	    }
	    Vector solutions = generateConflictSol(schedulesToArray(fSensInfo),new Vector(fSensInfo.keySet()), TMC.getObjFunc().getCurrObj(), TMC.getNumAlts());
	
	    Vector best = getBestSolution(solutions);
	    Hashtable currSolution = toSensorMajor(best, new Vector(), TMC.getName());
	    bindSolution(sensScheds, currSolution);
	}

	log.log("Random answer!!!!!! Util: " + calculateUtility(sensScheds, mans) + " Tracks: " +  calculateTotTracks(sensScheds, mans), log.LOG_INFO);
	log.log("Final Scheds" + sensScheds.toString(), log.LOG_INFO);
	setProperty(PROP_RANDOMUTILITY, new Float(calculateUtility(sensScheds,mans)));
	setProperty(PROP_RANDOMTRACKS, new Integer(calculateTotTracks(sensScheds,mans)));
	return true;
    }

    private int calculateTotTracks(Hashtable sensScheds, Vector mans){
	int totalTracks = 0;
	for(Iterator i = mans.iterator();i.hasNext();){
	    checkRun();
	    TMConflict TMC = (TMConflict) i.next();
	    int[] slotCounts = new int[slotCount];
	    for(Iterator j = TMC.getConflicts().iterator();j.hasNext();){
		SlotSchedule SS = (SlotSchedule) sensScheds.get(j.next());
		for(int slot = 0;slot<slotCount;slot++){
		    if(SS.hasCommitment(slot, TMC.getName()))
			slotCounts[slot] ++;
		}
	    }
	    for(int slot = 0;slot<slotCount;slot++){
		if(slotCounts[slot] > 2){
		    totalTracks ++;
		    break;
		}
	    }
	}
	return totalTracks;
    }

    private int calculateTrackable(Vector mans){
	int totalTracks = 0;
	for(Iterator i = mans.iterator();i.hasNext();){
	    checkRun();
	    TMConflict TMC = (TMConflict) i.next();
	    if(TMC.getNumAlts() >2)
		totalTracks++;	    
	}
	return totalTracks;
    }


    private float calculateUtility(Hashtable sensScheds, Vector mans){
	float totalUtil = 0.0f;
	for(Iterator i = mans.iterator();i.hasNext();){
	    checkRun();
	    TMConflict TMC = (TMConflict) i.next();
	    int[] slotCounts = new int[slotCount];
	    for(Iterator j = TMC.getConflicts().iterator();j.hasNext();){
		SlotSchedule SS = (SlotSchedule) sensScheds.get(j.next());
		for(int slot = 0;slot<slotCount;slot++){
		    if(SS.hasCommitment(slot, TMC.getName()))
			slotCounts[slot] ++;
		}
	    }
	    for(int slot = 0;slot<slotCount;slot++){
		if(slotCounts[slot] > 2){
		    totalUtil += (float) Math.sqrt(slotCounts[slot]-2);
		}
	    }
	}
	return totalUtil - 2.0f * (calculateTrackable(mans) - calculateTotTracks(sensScheds, mans));
    }

    /**
     * generateConflictSol is responsible for, given sensor schedules,
     * determining if a non-conflicting solution exists on all or a subset
     * of the sensor and slots that are unser consideration
     * If there is then it returns a vector of all of the non-conflicting
     * solutions if none, then it return null
     * <p>
     * There are several ways to use this function that have value
     *
     * 1.  If the sensors (in schedules) have elements that are 
     * occupied, this function will take that into account.  
     * This can be used to generate the set of acceptable 
     * solutions for local resolution
     * In this use, you must specify that the availSensors = 
     * schedules.size() in order to get solutions for all of
     * the sensors
     *
     * 2.  If you want to generate the set of acceptable
     * solutiond for a set of sensors and want it to be
     * non-respective of the sensors schedules then create
     * a hashtable with the agent names as keys and having
     * empty slot schedules as elements.  Again you must
     * specify the availSensors = schedules.size()
     *
     * 3.  If you want to just get the the possible solutions
     * for 2 out of 4 sensors then schedules should only
     * contain the names and schedules of the 2 sensors you want
     * schedules for.  The availSensors should be set to 4 in this
     * case.  This will generate the combinations respective of the
     * schedules you have listed.
     *
     *
     * @param schedules key: sensore name  element: SlotSchedule
     * @param obj The current objective level
     * @return Vector (solutions) elements: Vector (slots) elements: 
     *  Vector (sensors) elements: agent names
     *     
     */
    Vector generateConflictSol(int[][] schedMap, Vector sensNames, ObjEntry obj, int availSensors) {



	//invert the schedules
	Vector slotInfo = toSlotMajor(schedMap, sensNames);

	int goodSlots = 0;
	Vector goodStuff = new Vector();
	for(int i =0;i<slotInfo.size();i++){
	    Vector tmpVect = (Vector) slotInfo.elementAt(i);
	    int count = tmpVect.size();
	    //log.log("slot i: " + i + " count: " + count + " data: " + tmpVect.toString(), Log.LOG_DEBUG);

	    if(availSensors-(sensNames.size()-count) >= obj.numAgents()){
		goodStuff.add(new Integer(i));
		goodSlots++;
	    }
	}

        checkRun();
	if(goodSlots >= obj.numSlots()){
	    //great, we know there is at least one good solution
	    //log.log("Found at least one good solution.  Generating combinations", Log.LOG_SPAM);

	    //first, let's get the list of combinations of good
	    //slots, this is used in generation of the legal solutions
	    
	    Vector slotCombs = Choose(goodStuff, obj.numSlots());


	    //slotComb now has vectors of all possible slot combinations 
	    //which meet the criteria for a solution

	    //now we need for each of those slots, the legal combinations
	    //of agents

	    Vector sensCombs = new Vector();	
	    for(int j=0;j<slotInfo.size();j++){
		//does this slot have enough to even consider it as a 
		//a part of a viable solution?
		checkRun();
		int numSensInSlot = ((Vector) slotInfo.elementAt(j)).size();
		int availForSlot = availSensors - (sensNames.size() - numSensInSlot);
		int lb = Math.max(0, obj.numAgents()-
				availForSlot+numSensInSlot);
		int ub = Math.min(numSensInSlot, obj.numAgents());
		//log.log("lb " + lb+ " ub " + ub, Log.LOG_SPAM);
		if(lb <= ub){
		    //the slot can generate at least one good combination
		    checkRun();		    
		    Vector slotSensCombs = new Vector();
		    //for each of the slots generate the useable 
		    //sensor combination
		    
		    for(int i=lb;i<=ub;i++){
		    //generate all combinations for the n sensors such that
		    //we choose i
			Vector sensComb = Choose((Vector) slotInfo.
						 elementAt(j),i);
			slotSensCombs.addAll(sensComb);
		    }
		    sensCombs.add(slotSensCombs);
		}
		else{
		    sensCombs.add(new Vector());
		}
	    }

	    //for(int i=0;i<sensCombs.size();i++){
	    //	log.log("Combination " + i + " is " + ((Vector) sensCombs.elementAt(i)).toString(), Log.LOG_SPAM);
	    //}


	    //now we nee to construct the final answer
	    //for each slot combination, we need to pair wise match the slot 
	    //combinations in the slots
	    Vector solution = new Vector();
	    
	    for(int i =0;i<slotCombs.size();i++){
		//get one of the slot combinations
		Vector slotComb = (Vector) slotCombs.elementAt(i);
		checkRun();		
		//log.log("Evaluating slot combination " + i + ": " + slotComb.toString(), Log.LOG_SPAM);


		//for this slot combination we need to construct
		//a temporary sensory combination list with only the
		//slots in the combination
		//for all the other slots, we need to add a record
		//so the table join works correctly
		
		Vector tmpSensCombs = new Vector();
		//fill tmpSensCombs with empty vectors
		for(int j =0;j<sensCombs.size();j++){
		    tmpSensCombs.add(new Vector());
		}
		checkRun();		
		for(int j =0;j<slotComb.size();j++){
		    int slotNum = ((Integer) slotComb.elementAt(j)).intValue();
		    tmpSensCombs.setElementAt(sensCombs.elementAt(slotNum), slotNum);
		    //log.log("Adding slot "+slotNum+" which is " + 
		    //    ((Vector) sensCombs.elementAt(slotNum)).toString()
		    //    + " to tmpSensCombs", Log.LOG_SPAM);
		}

		Vector tmpsol = tableJoin(tmpSensCombs);

		solution.addAll(tmpsol);
	    }

	    return solution;
	}
	return new Vector();
    }

    Vector tableJoin(Vector tables){
	Vector retData = new Vector();
	maxJoin(tables, 0, new Vector(), retData);
	return retData;
    }

    void maxJoin(Vector tables, int currdepth, Vector solution, Vector Sets){
	//this function computes the pair wise join of all of the vectors
	//in the vector table.  It returns tha answer as a vector of vectors

	Vector thisTable = (Vector) tables.elementAt(currdepth);
	int maxdepth = tables.size();
        checkRun();
	if(thisTable.size() == 0){
	    solution.add(new Vector());
	    if(currdepth < maxdepth-1){
		maxJoin(tables, currdepth +1, solution, Sets);
		solution.remove(currdepth);
	    }
	    else{
		Sets.add(new Vector(solution));
		solution.remove(currdepth);
	    }	
	}
	else{
	    for(int i=0;i<thisTable.size();i++){
		solution.add(thisTable.elementAt(i));
		if(currdepth < maxdepth-1){
		    maxJoin(tables, currdepth +1, solution, Sets);
		    solution.remove(currdepth);
		}
		else{
		    Sets.add(new Vector(solution));
		    solution.remove(currdepth);
		}
	    }
	}
    }

    Vector Choose(Vector elements, int r){
	//this function takes a vector of n elements and returns a vector
	//that contains the nCr possible combinations (vectors) of the elements

	Vector retData = new Vector();
	if(r>0){
	    nCr(r, new Vector(), 0, elements, retData);
	}
	else{
	    retData.add(new Vector());
	}
	return retData;
    }
	

    void nCr(int maxdepth, Vector solution, int first, 
	     Vector elements, Vector Sets){

	int currdepth = solution.size();
	int last = elements.size();
        checkRun();
	for(int i = first;i<= last-maxdepth+currdepth;i++){
	    
	    solution.add(elements.elementAt(i));
	    if(currdepth < maxdepth-1){
		nCr(maxdepth, solution, i+1, elements, Sets);
		solution.remove(currdepth);
	    }
	    else{
		//Operate on solution
		Sets.add(new Vector(solution));
		solution.remove(currdepth);
	    }
	}
    }

    Hashtable toSensorMajor(Vector slotData, Vector sensors, String manData){
	/*this function returns a hashtable with each key being a sensor
	  name and each element being a SlotSchedule

	  If you specify names in a vector, the hashtable that is returned
	  contain one element per name.  If the name was not in slotdata
	  the slotschedule associated with the sensor will be empty

	  coverts        
	
       	  Slot 1 S1, S3
	  Slot 2 S2, S3
	  Slot 3 S1, S3
	  
	  to

	  S1    None  T1   None
	  S2    T1   None   T1
	  S3    None None  None

	*/

	Hashtable tmpHash = new Hashtable();
 	for(int i=0;i<sensors.size();i++){
	    SlotSchedule SS = new SlotSchedule();
	    tmpHash.put(sensors.elementAt(i),SS);
	}
        checkRun();	
	for(int i=0;i<slotData.size();i++){
	    Vector names = (Vector) slotData.elementAt(i);
	    for(int j = 0;j<names.size();j++){
		SlotSchedule SS;
		if(tmpHash.containsKey(names.elementAt(j))){
		    //update the current slot schedule for that
		    //sensor
		    SS = (SlotSchedule) 
			tmpHash.get(names.elementAt(j));
		    
		}
		else{
		    //create a new entry in the hashtable and
		    //set the slot schedule correctly
		    SS = new SlotSchedule();
		    tmpHash.put(names.elementAt(j), SS);
		}
		checkRun();
		SS.add(i,manData);
	    }
	}
	return tmpHash;
    }

    /**
     * This function converts the map and the names into
     * a list for each slot of available resources
     */
    Vector toSlotMajor(int[][] schedMap, Vector names){
	//this function returns a vector of the slots where each slot is
	//a vector that contains the names of the sensors that have the slot
	//available

	/*converts        
	
	  S1    None  T1   None
	  S2    T2   None   T2
	  S3    None None  None


	  to

	  Slot 1 S1, S3
	  Slot 2 S2, S3
	  Slot 3 S1, S3
	
	*/

	Vector slotData = new Vector(SlotSchedule.NUMSLOTS);
	//count the columner counts
	for(int i = 0;i<SlotSchedule.NUMSLOTS;i++){
	    Vector sens = new Vector();
	    for(int j = 0;j<names.size();j++){
		String sensName = (String) names.elementAt(j);
		if(schedMap[j][i] == 0){
		    //free slot
		    sens.add(sensName);
		}
	    }
	    slotData.add(sens);
	}
	return slotData;
    }

    public int[][] schedulesToArray(Hashtable scheds){
	int[][] schedMap = new int[scheds.size()][SlotSchedule.NUMSLOTS];
	Vector names =  new Vector(scheds.keySet());
	for(int i = 0;i<names.size();i++){
	    String name = (String) names.elementAt(i);
	    SlotSchedule SS = (SlotSchedule) scheds.get(name);
	    for(int j = 0;j<SlotSchedule.NUMSLOTS;j++){
		schedMap[i][j] = SS.numCommitments(j);
	    }
	}
	return schedMap;
    }


    /**
     * Finds the manager who is causing the over-constrained condition
     * and lower his objective function
     * @param managers A Vector of the TMConflicts for the managers in the negotiation
     * @return True if one of the managers was lowered
     */
    boolean lowerObjective(Vector managers){

	//The method for doing this is simple, just find the
	//manager with both the highest level and that cannot meet
	//his needs with his own non con sensors and lower his level
	int man = findNextEligible(managers,0);
	for(int i = 0;i<man;i++){
	    ((TMConflict) managers.elementAt(i)).
		getObjFunc().highestObj();
	} 
        checkRun();
	while(man != -1){
	    TMConflict thisTM = (TMConflict) managers.elementAt(man);
	    if(thisTM.getObjFunc().lowerObj()){
		printObjs(managers);
		return true;
	    }
	    else{
		//reset mine
		thisTM.getObjFunc().highestObj();
		int thisManNum = man;
		man = findNextEligible(managers,man+1);
		for(int i = thisManNum;i<man;i++){
		    ((TMConflict) managers.elementAt(i)).
			getObjFunc().highestObj();
		} 
	    }
	}
	return false;
    }


    private int findNextEligible(Vector managers, int index){
        checkRun();
	for(int i=index;i<managers.size();i++){
	    TMConflict thisTM = (TMConflict) managers.elementAt(i);
	    ObjEntry obj = thisTM.getObjFunc().getCurrObj();
	    int numOwn = thisTM.getNumAlts() - thisTM.numLocalConflicts();
	    //check if eligible
	    if(obj.numAgents() > numOwn){
		//the first, try to lower his 
		return i;
	    }
	}
	return -1;
    }
    
    private Vector cloneVector(Vector vect){
	Vector ret = new Vector();
	for(int i=0;i<vect.size();i++){
	    ret.add(((TMConflict) vect.elementAt(i)).clone());
	}
	return ret;
    }

    private void printObjs(Vector mans){
	String outdata = "";
	for(Iterator i = mans.iterator();i.hasNext();){
	    TMConflict TMC = (TMConflict) i.next();
	    ObjEntry objE= TMC.getObjFunc().getCurrObj();
	    outdata = outdata + "Man: " + TMC.getName() + " " +  objE.toString() + ", ";
	}
	log.log(outdata, log.LOG_INFO);
    }

    private void setStartObjs(Vector mans){ 
	for(Iterator i = mans.iterator();i.hasNext();){
	    TMConflict TMC = (TMConflict) i.next();
	    setStartObj(TMC);
	}
    }

    private void setStartObj(TMConflict TMC){
	ObjFunction objective = TMC.getObjFunc();
	//make sure that we can start our objective level at something
	//that has the posibility of being achieveble
	ObjEntry objE = objective.getCurrObj();
	while(objE.numAgents() > TMC.getNumAlts()){
	    checkRun();
	    objective.lowerObj();
	    objE = objective.getCurrObj();
	}  
    }

    Vector orderMans(Vector managers){
	Vector tempVect = new Vector(managers.size());
	Vector manList = new Vector(managers.size());
	Random rnd = new Random();
	
	for(Iterator i=managers.iterator();i.hasNext();){
	    tempVect.add(i.next());
	}
	
	for(int i=tempVect.size();i>0;i--){
	    manList.add(tempVect.remove(rnd.nextInt(i)));
	}
	return manList;
    }

    private Vector getBestSolution(Vector solutions){
	Random rnd = new Random();
	return (Vector) solutions.elementAt(rnd.nextInt(solutions.size()));
    }    



    void bindSolution(Hashtable sensScheds, Hashtable solution){
	/*this method takes a solution in sensor major order
	  a track object and a hashtable of slot schedules
	  and updates the currrent track state with a new solution
	  as well as sends out new binding messages to enact the solution
	 */

	Enumeration e = solution.keys();
	while(e.hasMoreElements()){
	    String SName = (String) e.nextElement();
	    SlotSchedule SS = (SlotSchedule) solution.get(SName);
	    sensScheds.put(SName, SS);
	}
    }
}
