package farm.ants.agent;
import java.util.*;
/**
 * This class represents the objective function of a SPAM agent
 * <p>
 * Created: Mon Aug 12 09:52:22 2002
 *
 * @author Roger Mailler
 */
public class SPAMObjFunction implements ObjFunction {
    int currLevel = 0;
    Vector levels = new Vector();
    float nicePercent = 0.1f;
    int niceLevel = 0;

    /**
     * Constructor
     */
    public SPAMObjFunction() {
	createFunction();
    }

    /**
     * Gets the current objetive entry for this track
     * @return The current ObjEntry
     */
    public ObjEntry getCurrObj(){
	return (ObjEntry) levels.elementAt(currLevel);
    }

    /**
     * Sets the current objetive level for this track using an ObjEntry
     * @param obj The current ObjEntry
     */
    public void setCurrObj(ObjEntry obj){
	currLevel = levels.indexOf(obj);
	highestNiceObj();
    }

    /**
     * Lowers the objective level
     * @return True if successful
     */
    public boolean lowerObj(){
	if (currLevel < levels.size()-1){
	    currLevel++;
	    highestNiceObj();
	    return true;
	}
	else{
	    return false;
	}
    }
      
    /**
     * Increases the current objective level
     * @return True if successful
     */
    public boolean raiseObj(){
	if (currLevel > 0){
	    currLevel--;
	    highestNiceObj();
	    return true;
	}
	else{
	    return false;
	}
    }

    /**
     * Lowers the objective level to the lowest
     */
    public void lowestObj(){
	currLevel = levels.size() -1;
	highestNiceObj();
    }
    /**
     * Sets the current objective level to the highest 
     */
    public void highestObj(){
	currLevel = 0;
	highestNiceObj();
    }

    /**
     * Lowers the nice objective level by one
     * @return True if it was lowered successfully
     */
    public boolean lowerNiceObj(){
	int slack = Math.round((float)(levels.size() - currLevel)*nicePercent);
	if ((niceLevel < currLevel + slack) && (niceLevel < levels.size() -1)){
	    niceLevel++;
	    return true;
	}
	else{
	    return false;
	}
    }
    
    /** 
     * raises the nice objective level by one
     * @return True  if it was raised successfully
     */
    public boolean raiseNiceObj(){
	if (niceLevel > currLevel){
	    niceLevel--;
	    return true;
	}
	else{
	    return false;
	}
    }

    /**
     * Raises the nice level to the highst possible (equal to currObj)
     */
    public void highestNiceObj(){
	niceLevel = currLevel;
    }

    /** 
     * Lowers the nice level to the lowest possible
     * (currObj - NicePercent*CurrObj)
     */
    public void lowestNiceObj(){
	int slack = Math.round((float)(levels.size() - currLevel)*nicePercent);
	niceLevel = Math.min(currLevel + slack, levels.size()-1);
    }

    /**
     * Gets the current nice Objective entry
     * @return The nice objective
     */
    public ObjEntry getNiceObj(){
	return (ObjEntry) levels.elementAt(niceLevel);
    }

    /**
     * Set the NicePercentage
     * @param percent The percentage of deviation for the
     * current objective level before needing to renegotiate
     */
    public void setNicePercent(float percent){
	nicePercent = percent;
    }





    /**
     * Constructs a default objective function
     */ 
    private void createFunction(){
	ObjEntry obj = new ObjEntry("4*3");
	levels.add(obj);
	obj = new ObjEntry("3*3");
	levels.add(obj);
    	obj = new ObjEntry("4*2");
	levels.add(obj);
    	obj = new ObjEntry("3*2");
	levels.add(obj);
     	obj = new ObjEntry("4*1");
	levels.add(obj);
    	obj = new ObjEntry("3*1");
	levels.add(obj);
    	/*obj = new ObjEntry("2*3");
	levels.add(obj);
    	obj = new ObjEntry("2*2");
	levels.add(obj);
    	obj = new ObjEntry("2*1");
	levels.add(obj);
    	obj = new ObjEntry("1*3");
	levels.add(obj);
    	obj = new ObjEntry("1*2");
	levels.add(obj);
    	obj = new ObjEntry("1*1");
	levels.add(obj);*/
     	obj = new ObjEntry("0*0");
	levels.add(obj);

    }

    /**
     * Returns a string represenation of the function
     * @return The string
     */
    public String toString(){
	String ret = "";
	for (int i=0;i<levels.size();i++){
	    ret = ret + ((ObjEntry) levels.elementAt(i)).toString() + ((i<levels.size()-1)?",":"");
	}
	return ret;
    }

} // SPAMObjFunction
