package farm.ants.agent;
import farm.metagent.*;
import utilities.Log;
import java.util.*;

/**
 * The resource monitoring PAM used by the SPAMAgent.  This machine
 * checks for alterations to the resources availablity to include add and
 * removes of the appropriatness of a resources and periodic checks
 * of conflicts on resources currently in use.
 * <p>
 * Created: Mon Jul 15 15:46:56 2002
 *
 * @author Roger Mailler
 */
public class TrackPAM extends PulseActionMachine {
    Log log = null;
    Agent me = null;
    String lastRes = "";
    int trackUpdate = 200;  //how many miliseconds to wait between
                            //checks of the resource requirments

    int SPAMSlack = 3000; //how many milliseconds before SPAM times out
    int maxAgents = 4;     //total number of agents for SPAM to consider
    long nextSPAMRun = 0;   //the next time the allocation should be checked
    int SPAMRefresh = 800;  //how many millisecond between
                             //casual examination for new conflicts

    Random rnd = new Random();

    /**
     * Constructor
     */
    public TrackPAM(Agent m) {
	me = m;
	log = Log.getDefault();
	setWaitMess(0);
	setWaitFor(WF_NONE);
	setAction("Track");
    }

    public boolean executeState(int stateNum){
	switch (stateNum){
	case 0:
	    return executeState0();
	default:
	    log.log("Unknown Track state code:" + stateNum, 
	    	    log.LOG_ERR);
	    return true;
	}
    }

    private boolean executeState0(){
	log.log("Executing Track State 0", log.LOG_INFO);
	boolean propagate = false;

	//simple machine, just get the current resource needs
	//and spawn a SPAM PAM if needed
	
	//check the requirements for the target
	String res = (String) me.getProperty(me.getAgentName() + ":res");
	int time = me.getTime();
	boolean neg = false;
	//log.log(me.getAgentName() + ":res",Log.LOG_INFO);
	Track tmpTrack = (Track) getAttribute("Track");


	Vector agentData = new Vector();
	StringTokenizer tok = new StringTokenizer(res, ",");
	while(tok.hasMoreTokens())  
	    agentData.add(new SensorData(tok.nextToken()));
	agentData = getBest(agentData, maxAgents);

	//log.log(res, log.LOG_INFO);
	if(res!=null && !res.equals(lastRes)){
	    //spawn a new SPAM PAM
	    if(!tmpTrack.contextEquals(agentData)){
		neg = true;
		tmpTrack.getObjFunc().highestObj();
		propagate = false;
	    }
	}
	else if((me.getTime() >= nextSPAMRun)){
	    if(checkForConflicts(tmpTrack) || 
	       checkForLocalImprovement(tmpTrack)){
		neg = true;
		propagate = true;
	    }
	    else{
		nextSPAMRun = me.getTime() + rnd.nextInt(SPAMRefresh-trackUpdate) + trackUpdate;
	    }
	}
	
	if(neg){
	    //log.log("Spawning a new SPAM Machine", log.LOG_INFO);
	    SPAM newNeg = new SPAM(me);
	    tmpTrack.setContext(agentData);
	    newNeg.setAttribute("Track", tmpTrack);
	    newNeg.setAttribute("Sensors", agentData);
	    log.log(agentData.toString(), log.LOG_INFO);
	    newNeg.setAttribute("Timeout", new Integer(me.getTime()+SPAMSlack));
	    newNeg.setAttribute("Parent", this);	
	    newNeg.setAttribute("propagate", new Boolean(propagate));
	    Controller.register(newNeg);
	    nextSPAMRun = me.getTime() + rnd.nextInt(SPAMRefresh-trackUpdate) + trackUpdate;
	    lastRes = res;
	    transition(0, WF_NEGOTIATION, 0, 0);
	    
	}
	else{
	    transition(0, WF_TIME, 0, me.getTime() + trackUpdate);
	}
	return false;
    }

    boolean checkForConflicts(Track track){
	Vector agentNames = track.getAgents();
	for(Iterator i = agentNames.iterator();i.hasNext();){
	    String agentName = (String) i.next();
	    if(track.hasSchedule(agentName)){
		SlotSchedule SS = track.getSchedule(agentName);
		for(int j = 0;j<SlotSchedule.NUMSLOTS;j++){
		    if(SS.hasCommitment(j,me.getAgentName()) && SS.numCommitments(j) > 1)
			return true;
		}
	    }
	}
	return false;
    }

    boolean checkForLocalImprovement(Track track){
	log.log("Checking for a local improvement", log.LOG_INFO);



	ObjEntry currObj = track.getObjFunc().getCurrObj();
	ObjFunction objFunc =  track.getObjFunc();
	int[] slotCounts =new int[SlotSchedule.NUMSLOTS];
	
	Vector agentNames = track.getAgents();
        for(Iterator i = agentNames.iterator();i.hasNext();){
            String agentName = (String) i.next();
            if(track.hasSchedule(agentName)){
                SlotSchedule SS = track.getSchedule(agentName);
		log.log("Agent " + agentName + " Sched: " + SS.toString(),log.LOG_INFO);
                for(int j = 0;j<SlotSchedule.NUMSLOTS;j++){
                    if((SS.hasCommitment(j,me.getAgentName()) &&  SS.numCommitments(j) == 1) || SS.numCommitments(j) == 0)
			slotCounts[j]++;
                }
            }
        }
	for(int j = 0;j<SlotSchedule.NUMSLOTS;j++)
	    log.log("slotCounts[" + j+ "]: " + slotCounts[j],log.LOG_INFO);

	
	ObjEntry newBest = objFunc.getCurrObj();
	while(objFunc.raiseObj()){
	    ObjEntry newObj = objFunc.getCurrObj();
	    log.log("Checking obj level to  " + newObj.toString(),log.LOG_INFO);
	    int numAgents = newObj.numAgents();
	    int numSlots = newObj.numSlots();
	    
	    for(int j = 0;j<SlotSchedule.NUMSLOTS;j++){
		if(slotCounts[j] >= numAgents)
		    numSlots --;
	    }

	    if(numSlots <= 0){
		log.log("Changing newBest to  " + newObj.toString(),log.LOG_INFO);
		newBest = newObj;
	    }
	}

	log.log("Setting obj level to  " + newBest.toString(),log.LOG_INFO);
	objFunc.setCurrObj(newBest);

	if(currObj.equals(objFunc.getCurrObj())){
	    log.log("Leaving the OBJ Level at " + objFunc.getCurrObj().toString(),log.LOG_INFO);
	    return false;
	}
	else{
	    log.log("Changing the OBJ Level from " + currObj.toString() + " to " + objFunc.getCurrObj().toString(),log.LOG_INFO);
	    return true;
	}
    }

    private Vector getBest(Vector list, int num){
	//return the first num elements
	return new Vector(list.subList(0, Math.min(num,list.size())));
    }
} // TrackPAM
