package farm.ants.plugins; 
import java.rmi.*;
import java.rmi.server.*;
import farm.simulator.*;
import java.util.*;
import java.awt.*;
import javax.swing.*;

/**
 * Shows a display of the environment.  This GUI primarily shows sensors
 * (black) and targets (red) as they move about the environment.  It will
 * also show negotiations in progress (blue lines).  Sensor allocations
 * to targets are also shown with lines, and are differentiated based
 * on conflicted (orange) or conflict-free (yellow) status.  The width of
 * the allocation line indicates the amount of time (number of schedule 
 * slots) being given to the target by the sensor.
 */
public class FarmDisplay extends FarmPlugIn {
    
    int width = 500;
    int height = 500;
    boolean startUp = true;
    int numSensors = 50;
    int numTargets = 20;
    long seed = 1234567;
    Random rnd = new Random(seed);
    ArrayList sensors = new ArrayList();
    ArrayList targets = new ArrayList();
    FarmDisplayFrame display = null;
    boolean started = false;

    /**
     * Constructor
     */
    public FarmDisplay(String []args)throws RemoteException {
        super(args);

        // Command line args
        setPlugInName(getCommandLineArgument(args, CLA_NAME, "Farm Display"));
    }

    /**
     * This is called at the beginning of a pulse, before the
     * agents are started.
     */
    public void pulseBegin() throws RemoteException {
	if (startUp) {
	    if (getProperty("targets") == null) {
		return;
	    } else {
		//we can do something
		//System.err.println("Have data to get");
		startUp = false;

		width = ((Integer) getProperty("width")).intValue();
		height = ((Integer) getProperty("height")).intValue();
		numSensors = ((Integer) getProperty("sensors")).intValue();
		numTargets = ((Integer) getProperty("targets")).intValue();

		display.setWidth(width);
		display.setHeight(height);
		display.setSensors(createSensors(numSensors));
		display.setTargets(createTargets(numTargets));
	    }
	} else {
	    //update the targets locations
            if (timeChanged()) {
                display.updateDisplay();
            }
	}
    }

    /**
     * This is called at the end of a pulse, after the agents
     * should have completed.
     */
    public void pulseEnd() {
    }

    /**
     */
    private ArrayList createSensors(int numSens) {
	ArrayList sens = new ArrayList();
	for(int i = 0;i<numSens;i++){
	    String name = "SA" + i;
	    SensorModel sensMdl = new SensorModel(this, name);	    	    
	    sens.add(sensMdl);
	    //System.err.println(name + " " + sensMdl.getX(true));
	}
	return sens;
    }

    /**
     */
    private HashMap createTargets(int numTargs){
	HashMap targs = new HashMap();
	for(int i = 0;i<numTargs;i++) {
	    String name = "M" + i;
	    TargetModel targMdl = new TargetModel(this, name);	    	    
	    targs.put(name,targMdl);
	    //System.err.println(name + " " + sensMdl.getX(true));
	}
	return targs;
    }

    /**
     */
    public void begin() throws RemoteException {
	super.begin();
	display = new FarmDisplayFrame();
	display.pack();
	display.show();
    }

    /**
     * The main function
     */
    public static void main(String[] args) {

        try {
            FarmDisplay disp = new FarmDisplay(args);
            disp.run();

        } catch (Exception e) {
            System.err.println("Bind exception: " + e.getMessage());
            e.printStackTrace();
        }

        System.exit(0);
    }
    
} // FarmDisplay
