package farm.graphcolor.agent;

import farm.simulator.*;
import farm.metagent.*;
import farm.graphcolor.plugins.drivers.*;

import java.util.*;
import utilities.Log;

/**
 */
public class RandomAgent extends Agent {
    NodeModel model;
    Random random = new Random();
    Hashtable neighbors = new Hashtable();

    /**
     * Constructor
     * @param metagent A handle to the metagent this agent belongs to
     * @param m The NodeModel assigned to this agent
     */
    public RandomAgent(FarmMetAgent metagent, NodeModel m) {
	super(metagent, m.getName());

        model = m;
    }

    /**
     */
    public void pulse() {
	super.pulse();

        // What color am i?
        int mycolor = model.getColor();

        // Respond to messages
        for (Message msg = getFirstIncomingMessage(); msg != null; msg = getFirstIncomingMessage()) {
            checkRun();
	    if (msg.getData().startsWith(">")) {
		Integer c = Integer.valueOf(msg.getData().substring(1));
                neighbors.put(msg.getSource(), c);
            }
        }

        // Pick a new color
        int bestcolor = chooseBestColor();
        if (mycolor != bestcolor) {
            log.log(getAgentName() + " changing color from " + mycolor + " to " + bestcolor, Log.LOG_INFO);
            model.setColor(bestcolor);
            for (Iterator i = model.getNeighbors(); i.hasNext(); ) {
                sendMessage(new Message(getAgentName(), (String)i.next(), ">" + bestcolor));
                checkRun();
            }
        }

        endRun();
    }

    /**
     * Selects the best color, based on local knowledge
     */
    public int chooseBestColor() {
        int colors[] = new int[model.getPossibleColors()];
        int mycolor = model.getColor();
        int best = (mycolor < 0 ? 0 : mycolor);

        // Rate the colors
        for (Iterator i = model.getNeighbors(); i.hasNext(); ) {
            Integer neighbor = (Integer)neighbors.get((String)i.next());
            if (neighbor != null) {
                int nc = neighbor.intValue();
                if (nc >= 0) {
                    colors[nc]++;
                }
            }
        }

        // My color rules!
        if (mycolor >= 0) {
            if (colors[mycolor] == 0) return mycolor;
        }

        // My color sucks, look elsewhere
        for (int i = 0; i < colors.length; i++) {
            // If one is clearly better, choose it
            if (colors[i] < colors[best]) {
                best = i;
            // If it's as good as what I've got, then
            // I might switch if you're nice to me
            } else if (colors[i] == colors[best] &&
                       random.nextInt(model.getPossibleColors()) == 0) {
                best = i;
            }
        }

        return best;
    }
}
