package farm.graphcolor.plugins.drivers; 

import farm.simulator.*;

import java.util.*;
import java.io.*;

/**
 * A model for describing a graph node.  Has accessors
 * for the node's name, neighbors and color
 */
public class NodeModel implements Serializable {
    protected FarmPlugIn plugin;
    protected String name;
    protected int num;
    //protected ArrayList neighbors = new ArrayList();
    protected Hashtable neighbors = new Hashtable();
    protected int color = -1;
    protected int priority = 0;

    /**
     * Constructor
     * @param p Some plugin
     * @param n The node number
     * @param c Initial color
     */
    public NodeModel(FarmPlugIn p, int n, int c) {
        plugin = p;

        num = n;
        name = "ND" + num;

        setColor(c);
    }

    /**
     * Constructor
     * @param p Some plugin
     * @param n The node number
     */
    public NodeModel(FarmPlugIn p, int n) {
        this(p, n, -1);
    }

    /**
     * Constructor
     * @param p Some plugin
     * @param s String to create from
     */
    public NodeModel(FarmPlugIn p, String s) {
        plugin = p;
	this.fromString(s);
    }

    /**
     * Returns the node's name
     */
    public String getName() { return name; }

    /**
     * Returns the node's number
     */
    public int getNumber() { return num; }

    /**
     * Returns the node's color
     */
    public int getColor() { return color; }

    /**
     * Sets the node's color
     */
    public void setColor(int c) { 

        if (color != c) {
            color = c;
            plugin.setProperty(name + ":color", new Integer(color)); 
        }
    }

    /**
     * Returns the total possible number of colors
     */
    public int getPossibleColors() { 

        Integer i = (Integer)plugin.getProperty(GraphColorDriver.CLA_COLORS);

        return i.intValue();
    }

    /**
     * Returns the node's priority
     */
    public int getPriority() { return priority; }

    /**
     * Sets the node's priority
     */
    public void setPriority(int p) { 

        if (priority != p) {
            priority = p;
            plugin.setProperty(name + ":priority", new Integer(priority)); 
        }
    }

    /**
     * Returns the names of the node's neighbor nodes
     */
    public Iterator getNeighbors() {
        neighbors = (Hashtable)plugin.getProperty(name + ":neighbors");
	if(neighbors == null) return new ArrayList().iterator();
        return neighbors.keySet().iterator();
    }

    /**
     * Returns the number of neighbors this node has in the graph
     */
    public int numNeighbors() {
        neighbors = (Hashtable)plugin.getProperty(name + ":neighbors");
	if(neighbors == null) return 0;
	return neighbors.size();
    }

    /**
     * Adds a neighbor node
     * @param name The name of the new neighbor
     */
    public void addNeighbor(String n) {
	addNeighbor(n, 1);
    }

    public void addNeighbor(String n, int val){
	if (n != null) {
            neighbors.put(n, new Integer(val));
        }
	
        plugin.setProperty(name + ":neighbors", neighbors);      
    
    }

    public int getNeighborValue(String n){
        neighbors = (Hashtable)plugin.getProperty(name + ":neighbors");
	if(neighbors.containsKey(n)){
	    return ((Integer) neighbors.get(n)).intValue();
	}
	return -1;
    }

    /**
     * Stringify
     */
    public String toString() {
        return getName() + " (" + getColor() + ") > " + neighbors; 
    }

    /**
     * Reverse Stringify
     */
    public void fromString(String str) {
	StringTokenizer tok = new StringTokenizer(str, "() >[],");
	name = tok.nextToken();
	num = Integer.parseInt(name.substring(2));
	setColor(Integer.parseInt(tok.nextToken()));
	while(tok.hasMoreTokens())
	    addNeighbor(tok.nextToken());
    }
}
