package farm.sat.plugins;

import java.rmi.*;
import java.rmi.server.*;

import utilities.Log;
import farm.simulator.*;
import farm.generic.plugins.*;
import farm.sat.plugins.drivers.*;

import java.util.*;

/**
 */
public class SATAnalysis extends FarmPlugIn {

    /**
     * Constructor
     * @param address The address of the remote simulator to eonnect to
     */
    public SATAnalysis(String []args)throws RemoteException {
        super(args);

        // Command line args
        setPlugInName(getCommandLineArgument(args, CLA_NAME, "SAT Analysis"));
    }

    /**
     * This is called at the end of a pulse.
     */
    public void pulseEnd() {

        try {
            LocalFunction function = new SATAnalysisFunction();            
            runLocalFunction(new SATAnalysisFunction());
            //function.run(simulator);

        } catch (RemoteException e) {
            System.err.println("Exception: " + e.getMessage());
            e.printStackTrace();
        }
    }

    /**
     * The main function
     */
    public static void main(String[] args) {

        try {
            FarmPlugIn plugin = new SATAnalysis(args);
            plugin.run();
        } catch (RemoteException e) {
            System.err.println("Exception: " + e.getMessage());
            e.printStackTrace();
        }
    }
}

class SATAnalysisFunction extends AnalysisFunction {

    // Constants
    public static final String PROP_TOTALVARIABLES = "TotalVariables";
    public static final String PROP_TOTALCLAUSES = "TotalClauses";
    public static final String PROP_TOTALCONFLICTS = "TotalConflicts";

    public Object run(PropertyOwner owner) {
	
        try {
            // Track messaging
            countMessagesSent(owner);
            countMessagesReceived(owner);
	    
            // Find clauses
            Collection clauses = (Collection)owner.getProperty(SATDriver.PROP_CLAUSES);
            owner.setProperty(PROP_TOTALCLAUSES, new Integer(clauses.size()));

            // Find clauses and values
            Map values = new HashMap();
            Collection list = owner.findProperties(":value");
            for (Iterator i = list.iterator(); i.hasNext(); ) {
                String name = (String)i.next();
                name = name.substring(0, name.indexOf(":"));

                Boolean b = (Boolean)owner.getProperty(name + ":value");
                values.put(name, b);
            }
            owner.setProperty(PROP_TOTALVARIABLES, new Integer(values.size()));

            // Count the satisfaction
            int unsatisfied = 0;
            for (Iterator i = clauses.iterator(); i.hasNext(); ) {
                Clause c = (Clause)i.next();
                if (!c.isSatisfiedBy(values)) {
                    unsatisfied++;
                }
            }

            owner.setProperty(PROP_TOTALCONFLICTS, new Integer(unsatisfied));

        } catch (RemoteException e) {
            System.err.println("Exception: " + e.getMessage());
            e.printStackTrace();
        }

        return null;
    }
}
