package farm.sat.plugins.drivers; 

import java.util.*;
import java.io.*;

public class Clause implements Serializable {
    Collection terms = new TreeSet();

    public Clause() {
    }

    public Clause(String s) {

        for (StringTokenizer tok = new StringTokenizer(s); tok.hasMoreTokens(); ) {
            String term = tok.nextToken();
            if (term.startsWith("-")) {
                addTerm(true, term.substring(1));
            } else {
                addTerm(false, term);
            }
        }
    }

    public Iterator getTerms() {
        return terms.iterator();
    }

    public void addTerm(boolean not, String var) {
        terms.add(new Term(not, var));
    }

    public boolean containsVariable(String var) {

        for (Iterator i = getTerms(); i.hasNext(); ) {
            Term t = (Term)i.next();            
            if (t.variable.equals(var)) return true;
        }
        return false;
    }

    public boolean equals(Object o) {
        Clause c = (Clause)o;

        return (terms.equals(c.terms));
    }

    public boolean isSatisfiedBy(Map m) {
        
        for (Iterator i = getTerms(); i.hasNext(); ) {
            Term t = (Term)i.next();
            Boolean value = (Boolean)m.get(t.variable);
            if (value != null) {
                if (t.not && value.booleanValue()) return false;
                else if (!t.not && !value.booleanValue()) return false;
            }
        }

        return true;
    }

    public String toString() {
        StringBuffer buf = new StringBuffer();
        for (Iterator i = getTerms(); i.hasNext(); ) {
            Term t = (Term)i.next();
            buf.append(t + (i.hasNext() ? " " : ""));
        }
        return buf.toString();
    }

    public String toDisplayString() {
        StringBuffer buf = new StringBuffer();
        for (Iterator i = getTerms(); i.hasNext(); ) {
            Term t = (Term)i.next();
            buf.append(t + (i.hasNext() ? " or " : ""));
        }
        return buf.toString();
    }
}
