/**
 */

package farm.metagent;

import java.io.*;

/**
 * A message object, for communicating ideas.  It is generally used
 * by Agents thusly:
 * <pre>
 *    sendMessage(new Message("Me", "You", "You smell of cabbage"));
 * </pre>
 * <p>
 * Timing information, from the agent's perspective, is usually
 * automatically generated by the send and recieve functions in
 * the agent.  The actual send and receive times are typically 
 * calculated and recorded by the metagent.
 *
 * @see Agent#sendMessage
 * @see FarmMetAgent#prepareMessageDelivery
 */
public class Message implements Serializable {
    protected String sorc;
    protected String dest;
    protected String data;
    private int sendtime, receivetime;
    private int asendtime, areceivetime;
    private boolean corrupted = false;

    /**
     * Constructor
     * @param source The name of the source of the message
     * @param dest The name of the destination of the message
     * @param data The data to send
     */
    public Message(String source, String dest, String data) {
        this.sorc = source;
        this.dest = dest;
        this.data = data;
    }

    /**
     * Accessors
     */
    public String getSource() { return sorc; }
    public String getDestination() { return dest; }
    public String getData() { return data; }

    public void setSendTime(int s) { sendtime = s; }
    public int getSendTime() { return sendtime; }
    public void setReceiveTime(int s) { receivetime = s; }
    public int getReceiveTime() { return receivetime; }

    public void setActualSendTime(int s) { asendtime = s; }
    public int getActualSendTime() { return asendtime; }
    public void setActualReceiveTime(int s) { areceivetime = s; }
    public int getActualReceiveTime() { return areceivetime; }

    public boolean isCorrupted() { return corrupted; }
    public void setCorrupted(boolean b) { corrupted = b; }

    /**
     * Stringify
     */
    public String toString() {

        return "S:" + getSource() + "(" + getSendTime() + ":" + getActualSendTime() + ") D:" + getDestination() + "(" + getReceiveTime() + ":" + getActualReceiveTime() + ") \"" + getData() + "\"";
    }
}
