/**
 */

package farm.simulator;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.Collection;

/**
 * The general plugin interface which is available to the simulator.
 * A minimal farm-compliant plugin must implement each of these
 * methods.
 * <p>
 * Because nearly all farm plugins are derived from FarmPlugIn (which
 * implements this interface), you generally don't need to worry about
 * the PlugIn interface itself.
 * @see FarmPlugIn
 */
public interface PlugIn extends Remote, PropertyOwner {

    public void setPulse(int pulse) throws RemoteException;    
    public void setPulse(int pulse, int time) throws RemoteException;    
    public void setStop() throws RemoteException;    

    public String getPlugInName() throws RemoteException;

    // Property accessors
    public void ownProperty(String name, Object value, Collection pushees) throws RemoteException;
    public Object disownProperty(String name) throws RemoteException;
    public DataToken getPropertyToken(String name) throws RemoteException;
    public void cacheProperty(String name, Object value, DataToken token) throws RemoteException;
    public void uncacheProperty(String name) throws RemoteException;
    public boolean startPushProperty(String name, String pluginname) throws RemoteException;
    public boolean stopPushProperty(String name, String pluginname) throws RemoteException;
    public Collection findProperties(String name) throws RemoteException;
}
