/**
 */

package farm.simulator;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.Collection;
import java.util.Iterator;

import farm.metagent.MetAgent;

/**
 * This interface must be implemented by the core simulation component
 * of farm.  It also serves as the interface of commands and data
 * available to remote connecting entities (such as plugins and metagents).
 * See the default farm simulator for more detailed information.
 * @see FarmSimulator
 */
public interface Simulator extends Remote, PropertyOwner {

    // Pulse types
    public static final int PULSE_NONE = 0;
    public static final int PULSE_BEGIN = 1;
    public static final int PULSE_IN = 2;
    public static final int PULSE_END = 3;

    // Constants
    public static final String CLA_SIMULATORHOST = "host";
    public static final String CLA_SIMULATORPORT = "port";
    public static final String P_TIME = "Time";
    public static final String NAME = "simulator";

    // MetAgent registers
    public void registerMetAgent(MetAgent agent) throws RemoteException;
    public void unregisterMetAgent(MetAgent agent) throws RemoteException;

    // PlugIn registers
    public void registerPlugIn(PlugIn plugin) throws RemoteException;
    public void unregisterPlugIn(PlugIn plugin) throws RemoteException;
    public PlugIn findPlugIn(String name) throws RemoteException;

    // Property accessors
    public void setPropertyOwner(String name, String source, Collection pushees) throws RemoteException;

    // Functional methods
    public void setStop() throws RemoteException;
    public Collection getPlugInNames() throws RemoteException;
}
